#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
basic logo interpreter
[http://www.cs.berkeley.edu/~bh/usermanual]
"""

# imports #####################################################################

import sys
import cmd
import pickle

from action import *


# aliases #####################################################################

aliases = {
	'fd':'forward', 
	'bk':'backward',
	'lt':'left',
	'rt':'right',
	'c': 'color',
	'pu':'penup',
	'pd':'pendown',
	'cs':'clearscreen'
}


# logo interpreter ############################################################

class Interpreter(cmd.Cmd):
	"""simple logo interpreter"""
	
	def __init__(self):
		cmd.Cmd.__init__(self)
		
		self.prompt = '? '
		self.actions = []
	
	
	def precmd(self, line):
		"""looking for aliases."""
		try:
			command, args = line.split(None, 1)
		except ValueError:
			command, args = line, ""
		if command in aliases:
			line = "%s %s" % (aliases[command], args)
		return line
	
	def onecmd(self, line):
		try:
			stop = cmd.Cmd.onecmd(self, line)
		except ValueError:
			print "! wrong type of arguments"
			stop = False
		return stop
	
	def postcmd(self, stop, line):
		"""refreshs the display after each command"""
		if not stop:
			print self.actions
		return stop
	
	
	def do_forward(self, d):
		"""forward <d>"""
		self.actions.append(Forward(int(d)))
	
	def do_backward(self, d):
		"""backward <d>"""
		self.actions.append(Forward(-int(d)))
	
	def do_left(self, a):
		"""left <a>"""
		self.actions.append(Left(int(a)))
	
	def do_right(self, a):
		"""right <a>"""
		self.actions.append(Left(-int(a)))
	
	def do_home(self, args):
		"""home"""
		self.actions.append(Home())
	
	def do_color(self, arg):
		"""color <r> <g> <b>"""
		self.actions.append(Color(map(float, arg.split(None, 3))))
	
	def do_penup(self, args):
		"""penup"""
		self.actions.append(Pen(down=False))
	
	def do_pendown(self, args):
		"""pendown"""
		self.actions.append(Pen(down=True))
	
	def do_clearscreen(self, args): 
		"""clearscreen"""
		self.actions = []
	
	
	def do_exit(self, args):
		print "bye!"
		return -1
		
	def do_EOF(self, args):
		print
		return self.do_exit(args)
	
	
	def do_dump(self, name):
		"""dumps the actions for debugging purpose"""
		if name == "": name = 'test'
		try:
			f = open('%s.logo' % name, 'a')
		except IOError:
			print "Unable to open %s.logo for writing" % name
		else:
			pickle.dump(self.actions, f)
			f.close()
	
	def do_load(self, name):
		"""loads the actions for debugging purpose"""
		if name == "": name = 'test'
		try:
			f = open('%s.logo' % name, 'r')
		except IOError:
			print "Unable to open %s.logo for reading" % name
		else:
			self.actions = pickle.load(f)
			f.close()
		
		
# test #########################################################################

def _test(argv=None):
	if argv is None: argv = sys.argv
	logo = Interpreter()
	return logo.cmdloop()


if __name__ == '__main__':
	sys.exit(_test())
